@@ -21,23 +21,21 @@ quantizeml main command-line interface.
 import argparse
 import json
 import os
-import re
-import numpy as np
 import os.path
-
+import re
 from importlib import metadata
 
-from keras.models import Model, Sequential
+import numpy as np
+from tf_keras.models import Model, Sequential
 
 from . import load_model, save_model
 from .layers import QuantizationParams
-from .models import quantize, dump_config
+from .models import dump_config, quantize
 from .models.transforms import insert_rescaling
-from .analysis.cli import main as analysis_cli, add_analysis_arguments
 
 
 def quantize_model(model_path, quant_config, qparams, samples, num_samples, batch_size, epochs,
-                   output_name=None, quantize_until=None):
+                   output_name=None, quantize_until=None, input_shape=None):
     """ CLI entry point to quantize a model using the provided configuration.
 
     Args:
@@ -52,6 +50,8 @@ def quantize_model(model_path, quant_config, qparams, samples, num_samples, batc
         output_name (str, optional): name of the output quantized model. If none provided
             set a default name as <model>_<config>.h5. Defaults to None.
         quantize_until (str, optional): Layer/node name up to which to quantize. Defaults to None.
+        input_shape (list or tuple, optional): A list or tuple specifying the new model input shape
+            excluding batch dimension. Defaults to None.
     """
     # Build name for the output model
     model_name = os.path.splitext(model_path)[0]
@@ -91,7 +91,7 @@ def quantize_model(model_path, quant_config, qparams, samples, num_samples, batc
     print(f"Quantizing model {model_path} with configuration file {quant_config}.")
     model_q = quantize(model, q_config=config, qparams=qparams, samples=samples_arr,
                        num_samples=num_samples, batch_size=batch_size, epochs=epochs,
-                       quantize_until=quantize_until)
+                       quantize_until=quantize_until, input_shape=input_shape)
     output_name = save_model(model_q, output_name)
     print(f"Saved quantized model to {output_name}.")
 
@@ -139,6 +139,34 @@ def insert_rescaling_and_save(model_path, dest_path, scale, offset):
     updated.save(dest_path)
 
 
+def add_analysis_arguments(parser):
+    asp = parser.add_subparsers(dest="analysis_action")
+
+    # Common arguments
+    a_parent_parser = argparse.ArgumentParser(add_help=False)
+    a_parent_parser.add_argument("-m", "--model", type=str, required=True, help="Model to analyze")
+
+    # Plot kernel distribution
+    k_parser = asp.add_parser("kernel_distribution", parents=[a_parent_parser],
+                              help="Plot kernel distribution")
+    k_parser.add_argument("-l", "--logdir", type=str, required=True,
+                          help="Log directory to save plots")
+
+    # Layer quantization error
+    qe_parser = asp.add_parser("quantization_error", parents=[a_parent_parser],
+                               help="Measure quantization error")
+    qe_parser.add_argument("mode", choices=["single", "cumulative", "weight"], default="single",
+                           help="Type of error to be computed. Defaults to %(default)s.")
+    qe_parser.add_argument("-fm", "--float_model", type=str, default=None,
+                           help="The base model (float version). Defaults to %(default)s.")
+    qe_parser.add_argument("-tl", "--target_layer", type=str, default=None,
+                           help="Compute per_channel error for a specific layer/node. "
+                           "Defaults to %(default)s.")
+    qe_parser.add_argument("-bs", "--batch_size", type=int, default=16,
+                           help="Batch size to generate samples. Defaults to %(default)s")
+    return [k_parser, qe_parser]
+
+
 def main():
     """ CLI entry point.
 
@@ -165,14 +193,20 @@ def main():
                           help="Quantization configuration file")
     q_parser.add_argument("-a", "--activation_bits", type=positive_int, default=8,
                           help="Activation quantization bitwidth")
+    q_parser.add_argument("-o", "--output_bits", type=positive_int, default=8,
+                          help="Output quantization bitwidth")
     q_parser.add_argument("--per_tensor_activations", action='store_true',
                           help="Quantize activations per-tensor")
     q_parser.add_argument("-w", "--weight_bits", type=positive_int, default=8,
                           help="Weight quantization bitwidth")
     q_parser.add_argument("-i", "--input_weight_bits", type=positive_int, default=8,
                           help="Input layer weight quantization bitwidth")
+    q_parser.add_argument("-id", "--input_dtype", type=str, default="uint8",
+                          help="Numpy-like dtype format to quantize the inputs")
     q_parser.add_argument("-b", "--buffer_bits", type=positive_int, default=32,
                           help="Buffer quantization bitwidth")
+    q_parser.add_argument("-lob", "--last_output_bits", type=int, default=None,
+                          help="Bitwidth of the model output")
     q_parser.add_argument("-s", "--save_name", type=str,
                           help="Name for saving the quantized model.")
     q_parser.add_argument("-sa", "--samples", type=str, default=None,
@@ -187,6 +221,12 @@ def main():
     q_parser.add_argument("-qu", "--quantize_until", type=str, default=None,
                           help="Layer/node name up to which to quantize. "
                           "By default the whole model is quantized.")
+    q_parser.add_argument("--input_shape",
+                          type=lambda x: tuple(map(int, x.split(','))),
+                          default=None,
+                          help="Shape to use for input_shape (Excluding batch dimension). "
+                          "Provide comma separated list for the shape. All values must be "
+                          "integers > 0. e.g. --input_shape 3,256,256 for CHW format.")
 
     # Dump config arguments
     c_parser = sp.add_parser(
@@ -211,18 +251,29 @@ def main():
     # Analysis actions
     a_parser = sp.add_parser("analysis",
                              help="Tool set to analyze potential models to be quantized")
-    add_analysis_arguments(a_parser)
+    a_parsers = add_analysis_arguments(a_parser)
 
     args = parser.parse_args()
 
     if args.action == "version":
         print(metadata.version('quantizeml'))
     elif args.action == "quantize":
+        if args.input_shape:
+            # Check that all elements are > 0
+            if any(value < 1 for value in args.input_shape):
+                raise ValueError(
+                    f"Invalid --input_shape: {args.input_shape}. "
+                    "Each dimension must be a greater than 0."
+                )
+
         qparams = QuantizationParams(activation_bits=args.activation_bits,
+                                     output_bits=args.output_bits,
                                      per_tensor_activations=args.per_tensor_activations,
                                      weight_bits=args.weight_bits,
                                      input_weight_bits=args.input_weight_bits,
-                                     buffer_bits=args.buffer_bits)
+                                     buffer_bits=args.buffer_bits,
+                                     input_dtype=args.input_dtype,
+                                     last_output_bits=args.last_output_bits)
         quantize_model(
             model_path=args.model,
             quant_config=args.quantization_config,
@@ -233,6 +284,7 @@ def main():
             epochs=args.epochs,
             output_name=args.save_name,
             quantize_until=args.quantize_until,
+            input_shape=args.input_shape
         )
     elif args.action == "config":
         dump_model_config(
@@ -242,4 +294,5 @@ def main():
     elif args.action == "insert_rescaling":
         insert_rescaling_and_save(args.model, args.dest_model, args.scale, args.offset)
     elif args.action == "analysis":
-        analysis_cli(args)
+        from .analysis.cli import main as analysis_cli
+        analysis_cli(a_parsers, args)
