@@ -19,11 +19,13 @@ __all__ = ["PaddedConv2D", "QuantizedConv2D", "QuantizedConv2DTranspose"]
 
 import tensorflow as tf
 import numpy as np
+import tf_keras as keras
 
-from keras import layers, backend
+from tf_keras import layers, backend
 
 from .layers_base import (register_quantize_target, rescale_outputs, tensor_inputs,
-                          neural_layer_init, register_aligned_inputs, QuantizedLayer)
+                          neural_layer_init, register_aligned_inputs, check_arg_constraints,
+                          QuantizedLayer)
 from .quantizers import AlignedWeightQuantizer
 from ..tensors import QTensor, QFloat, FixedPoint
 
@@ -41,6 +43,7 @@ def apply_padding(inputs, strides, kernel_size, padding_values):
         tf.Tensor or :obj:`QFloat`: padded inputs.
     """
     _, h, w, _ = inputs.shape
+    assert h is not None and w is not None, "The input height and width must be known."
     filter_width = kernel_size[0]
     filter_height = kernel_size[1]
     if h % strides[0] == 0:
@@ -120,7 +123,7 @@ def deconv_output_length(input_length, filter_size, padding,
     return length
 
 
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class PaddedConv2D(layers.Conv2D):
     """A convolutional layer that can use custom padding values.
 
@@ -135,21 +138,39 @@ class PaddedConv2D(layers.Conv2D):
 
     def __init__(self, *args, padding_value=None, **kwargs):
         super().__init__(*args, **kwargs)
-        self._padding_value = padding_value
         if padding_value is not None:
+            try:
+                self._padding_value = list(padding_value)
+            except TypeError:
+                self._padding_value = [padding_value]
             # When a custom padding_value is given, self.padding is overwritten and 'SAME' padding
             # will be done explicitly in the call.
             self.padding = 'valid'
+        else:
+            self._padding_value = None
+
+    @property
+    def padding_value(self):
+        # apply_padding perfoms legacy tf.pad when padding_value is a scalar.
+        # That is why we squeeze the list in order to prioritize the legacy op.
+        if self._padding_value is not None:
+            return np.squeeze(self._padding_value)
+
+    def build(self, input_shape):
+        # Ensure variables are build with the appropriate name, this is required for usage with
+        # Model.from_config that happens in sanitize
+        with tf.name_scope(self.name + '/'):
+            super().build(input_shape)
 
     def call(self, inputs):
         # We need a custom padding for specifics padding values
-        if self._padding_value is not None:
+        if self.padding_value is not None:
             # Raise an error if the padding values have not the expected number of values
-            pad_size = np.size(self._padding_value)
+            pad_size = np.size(self.padding_value)
             if pad_size not in (1, inputs.shape[-1]):
                 raise ValueError("The padding value must be a scalar or a list of values, for "
                                  f"a value per input channel. Receives {self._padding_value}.")
-            t_padding = tf.cast(self._padding_value, dtype=tf.float32)
+            t_padding = tf.cast(self.padding_value, dtype=tf.float32)
             # Note that we use the custom padding even when padding value provided is 0.
             inputs = apply_padding(inputs,
                                    list(self.strides),
@@ -168,9 +189,9 @@ class PaddedConv2D(layers.Conv2D):
         return config
 
 
-@register_quantize_target([layers.Conv2D, PaddedConv2D])
+@register_quantize_target([layers.Conv2D, PaddedConv2D], has_weights=True)
 @register_aligned_inputs
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class QuantizedConv2D(QuantizedLayer, layers.Conv2D):
     """A convolutional layer that operates on quantized inputs and weights.
 
@@ -183,20 +204,37 @@ class QuantizedConv2D(QuantizedLayer, layers.Conv2D):
             padding for the 'same' convolution type. Padding is per-tensor if one value is provided
             or per-channel otherwise. If None, zero-padding is used. Defaults to None.
     """
+    arg_constraints = {
+        'groups': 1
+    }
+
     @neural_layer_init(False)
     def __init__(self, *args, padding_value=None, quant_config=None, **kwargs):
+        check_arg_constraints(self, kwargs)
         # Limit buffer bitwidth to 27
         self.quant_config['buffer_bitwidth'] = min(28, self.quant_config['buffer_bitwidth'])
         self.buffer_bitwidth = self.quant_config['buffer_bitwidth'] - 1
 
-        self.padding_value = padding_value
         if padding_value is not None:
+            try:
+                self._padding_value = list(padding_value)
+            except TypeError:
+                self._padding_value = [padding_value]
             # We quantize the padding_value with the same bitwidth as the weights
             self.padding_value_quantizer = AlignedWeightQuantizer(self.weight_quantizer.bitwidth,
                                                                   signed=True)
             # When a custom padding_value is given, self.padding is overwritten and 'SAME' padding
-            # will be done explicitely in the call.
+            # will be done explicitly in the call.
             self.padding = 'valid'
+        else:
+            self._padding_value = None
+
+    @property
+    def padding_value(self):
+        # apply_padding perfoms legacy tf.pad when padding_value is a scalar.
+        # That is why we squeeze the list in order to prioritize the legacy op.
+        if self._padding_value is not None:
+            return np.squeeze(self._padding_value)
 
     @tensor_inputs([QTensor, tf.Tensor])
     @rescale_outputs
@@ -215,7 +253,7 @@ class QuantizedConv2D(QuantizedLayer, layers.Conv2D):
             pad_size = np.size(self.padding_value)
             if pad_size not in (1, inputs.shape[-1]):
                 raise ValueError("The padding value must be a scalar or a list of values, for "
-                                 f"a value per input channel. Receives {self.padding_value}.")
+                                 f"a value per input channel. Receives {self._padding_value}.")
             t_padding_value = tf.cast(self.padding_value, dtype=tf.float32)
             # Quantize and align padding_value on the inputs
             q_padding_value = self.padding_value_quantizer(t_padding_value, inputs)
@@ -248,14 +286,14 @@ class QuantizedConv2D(QuantizedLayer, layers.Conv2D):
 
     def get_config(self):
         config = super().get_config()
-        if self.padding_value is not None:
-            config["padding_value"] = self.padding_value
+        if self._padding_value is not None:
+            config["padding_value"] = self._padding_value
         return config
 
 
-@register_quantize_target(layers.Conv2DTranspose)
+@register_quantize_target(layers.Conv2DTranspose, has_weights=True)
 @register_aligned_inputs
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class QuantizedConv2DTranspose(QuantizedLayer, layers.Conv2DTranspose):
     """A transposed convolutional layer that operates on quantized inputs and weights.
 
