@@ -18,16 +18,16 @@
 __all__ = ["QuantizedDense"]
 
 import tensorflow as tf
-import keras
+import tf_keras as keras
 
 from .layers_base import (register_quantize_target, rescale_outputs, tensor_inputs,
                           neural_layer_init, register_aligned_inputs, QuantizedLayer)
 from ..tensors import QTensor
 
 
-@register_quantize_target(keras.layers.Dense)
+@register_quantize_target(keras.layers.Dense, has_weights=True)
 @register_aligned_inputs
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class QuantizedDense(QuantizedLayer, keras.layers.Dense):
     """A Dense layer that operates on quantized inputs and weights
 
