@@ -23,8 +23,10 @@ __all__ = ["QuantizedDepthwiseConv2D", "DepthwiseConv2DTranspose",
 
 
 import tensorflow as tf
-from keras import backend
-from keras.layers import DepthwiseConv2D
+import tf_keras as keras
+
+from tf_keras import backend
+from tf_keras.layers import DepthwiseConv2D
 
 from .layers_base import (register_quantize_target, rescale_outputs, tensor_inputs,
                           neural_layer_init, register_aligned_inputs, check_arg_constraints,
@@ -33,9 +35,9 @@ from .convolution import deconv_output_length
 from ..tensors import FixedPoint, QFloat
 
 
-@register_quantize_target(DepthwiseConv2D)
+@register_quantize_target(DepthwiseConv2D, has_weights=True)
 @register_aligned_inputs
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class QuantizedDepthwiseConv2D(QuantizedLayer, DepthwiseConv2D):
     """ A depthwise convolutional layer that operates on quantized inputs and weights.
 
@@ -75,7 +77,7 @@ class QuantizedDepthwiseConv2D(QuantizedLayer, DepthwiseConv2D):
         return outputs
 
 
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class DepthwiseConv2DTranspose(DepthwiseConv2D):
     """ A transposed depthwise convolutional layer.
 
@@ -92,6 +94,11 @@ class DepthwiseConv2DTranspose(DepthwiseConv2D):
         check_arg_constraints(self, kwargs)
         super().__init__(*args, **kwargs)
 
+    def build(self, input_shape):
+        # Ensure variables are build with the appropriate name
+        with tf.name_scope(self.name + '/'):
+            super().build(input_shape)
+
     def call(self, inputs):
         # Infer the dynamic output shape
         inputs_shape = tf.shape(inputs)
@@ -141,9 +148,9 @@ class DepthwiseConv2DTranspose(DepthwiseConv2D):
         return outputs
 
 
-@register_quantize_target(DepthwiseConv2DTranspose)
+@register_quantize_target(DepthwiseConv2DTranspose, has_weights=True)
 @register_aligned_inputs
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class QuantizedDepthwiseConv2DTranspose(QuantizedLayer, DepthwiseConv2DTranspose):
     """ A transposed depthwise convolutional layer that operates on quantized
     inputs and weights.
