@@ -18,8 +18,9 @@
 __all__ = ["QuantizedMaxPool2D", "QuantizedGlobalAveragePooling2D"]
 
 import tensorflow as tf
+import tf_keras as keras
 
-from keras.layers import MaxPool2D, GlobalAveragePooling2D
+from tf_keras.layers import MaxPool2D, GlobalAveragePooling2D
 
 from .layers_base import (register_quantize_target, register_no_output_quantizer, rescale_outputs,
                           tensor_inputs, apply_buffer_bitwidth, QuantizedLayer)
@@ -29,7 +30,7 @@ from ..tensors import FixedPoint, QTensor, QFloat
 
 @register_quantize_target(MaxPool2D)
 @register_no_output_quantizer
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class QuantizedMaxPool2D(QuantizedLayer, MaxPool2D):
     """A max pooling layer that operates on quantized inputs.
 
@@ -55,7 +56,7 @@ class QuantizedMaxPool2D(QuantizedLayer, MaxPool2D):
 
 
 @register_quantize_target(GlobalAveragePooling2D)
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class QuantizedGlobalAveragePooling2D(QuantizedLayer, GlobalAveragePooling2D):
     """A global average pooling layer that operates on quantized inputs.
 
