@@ -18,7 +18,7 @@
 __all__ = ["QuantizedFlatten"]
 
 import tensorflow as tf
-import keras
+import tf_keras as keras
 import functools
 import operator
 
@@ -31,7 +31,7 @@ from ...tensors import FixedPoint
 @register_quantize_target(keras.layers.Flatten)
 @register_no_output_quantizer
 @register_aligned_inputs
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class QuantizedFlatten(keras.layers.Flatten):
     """A Flatten layer that operates on quantized inputs
     """
