@@ -18,7 +18,7 @@
 __all__ = ["QuantizedPermute"]
 
 import tensorflow as tf
-import keras
+import tf_keras as keras
 
 from ..layers_base import (register_quantize_target, tensor_inputs, register_no_output_quantizer,
                            register_aligned_inputs)
@@ -29,7 +29,7 @@ from ...debugging import assert_equal
 @register_quantize_target(keras.layers.Permute)
 @register_no_output_quantizer
 @register_aligned_inputs
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class QuantizedPermute(keras.layers.Permute):
     """A Permute layer that operates on quantized inputs
 
