@@ -21,17 +21,19 @@ QuantizedSeparableConv2D layer definition.
 __all__ = ["QuantizedSeparableConv2D"]
 
 import tensorflow as tf
-from keras.layers import SeparableConv2D
-from keras import backend
+import tf_keras as keras
+
+from tf_keras.layers import SeparableConv2D
+from tf_keras import backend
 
 from .layers_base import (register_quantize_target, rescale_outputs, tensor_inputs,
                           neural_layer_init, register_aligned_inputs, QuantizedLayer)
 from ..tensors import FixedPoint
 
 
-@register_quantize_target(SeparableConv2D)
+@register_quantize_target(SeparableConv2D, has_weights=True)
 @register_aligned_inputs
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class QuantizedSeparableConv2D(QuantizedLayer, SeparableConv2D):
     """ A separable convolutional layer that operates on quantized inputs and weights.
 
