@@ -15,61 +15,40 @@
 # limitations under the License.
 # ******************************************************************************
 
-__all__ = ["calibrate", "calibration_required"]
+__all__ = ["calibrate"]
 
 import numpy as np
 import tensorflow as tf
 
 from copy import deepcopy
-from keras.layers import (Conv2D, Conv2DTranspose, SeparableConv2D, DepthwiseConv2D, Dense, Dropout,
-                          Reshape, Flatten, MaxPool2D, GlobalAvgPool2D)
+from tf_keras.layers import (Conv2D, Conv2DTranspose, SeparableConv2D, DepthwiseConv2D, Dense,
+                             Dropout, Reshape, Flatten, MaxPool2D, GlobalAvgPool2D, Rescaling)
 
+from .random import generate_keras_random_samples
 from .transforms.insert_layer import insert_in_config
-from .transforms.transforms_utils import get_layers_by_type
-from .utils import apply_weights_to_model, get_model_input_dtype
-from ..layers import (OutputQuantizer, OutputObserver, PaddedConv2D, DepthwiseConv2DTranspose,
+from .transforms.transforms_utils import get_layers_by_type, get_layer_index
+from .transforms.align_rescaling import _fold_rescaling
+from .utils import apply_weights_to_model
+from ..layers import (InputObserver, OutputObserver, PaddedConv2D, DepthwiseConv2DTranspose,
                       BufferTempConv, DepthwiseBufferTempConv, StatefulRecurrent,
-                      StatefulProjection, Dequantizer)
+                      StatefulProjection, InputQuantizer, Dequantizer, update_batch_size)
 from ..tensors import FixedPoint, floor_log2
 
 
-def calibration_required(model):
-    """Checks if a model requires calibration.
-
-    If one of the 'OutputQuantizer' layers in the model has its range_max variable set to 1, it
-    requires calibration.
-
-    Args:
-        model (keras.Model): the model to check
-
-    Returns:
-        bool: True if calibration is required, False otherwise.
-    """
-    calibrables = get_layers_by_type(model, OutputQuantizer)
-    for calibrable in calibrables:
-        # If the model has never been calibrated, all range_max of the OutputQuantizer objects
-        # will be set to 1.
-        if tf.reduce_all(calibrable.variables[0] == 1):
-            return True
-    # all calibrable objects are set
-    return False
-
-
 def _get_calibration_model(model, qmodel):
-    """Builds a calibration model with OutputObserver added between blocks.
+    """Builds a calibration model with Observers added.
 
     Args:
         model (keras.Model): input model
         qmodel (keras.Model): quantized model
 
     Returns:
-        keras.Model, dict: the calibration ready model and dict mapping end of block layer names to
-        the name of their OutputObserver.
+        keras.Model, dict: the calibration ready model and mapping of layer names with its Observer.
     """
     # Get model config to edit
     config = deepcopy(model.get_config())
 
-    # Insert OutputObservers where OutputQuantizers are
+    # Insert Observers
     end_of_blocks = {}
     for layer in qmodel.layers:
         if getattr(layer, 'out_quantizer', None):
@@ -78,6 +57,20 @@ def _get_calibration_model(model, qmodel):
             end_of_blocks[layer.name] = observer.name
             # Insert observer in config
             insert_in_config(model, layer.name, observer, config)
+        elif isinstance(layer, InputQuantizer):
+            # Search layer linked to InputQuantizer.
+            input_layer = layer.inbound_nodes[0].inbound_layers
+            # Build an observer and store it for future use.
+            # Note we specified its input shape in cases it is the first layer.
+            observer = InputObserver(axis=layer.axis, input_shape=input_layer.input_shape[0][1:])
+            end_of_blocks[layer.name] = observer.name
+            # Verify that model contains input_layer in config.
+            # Otherwise, InputObserver will be set in the input.
+            if (layer_id := get_layer_index(config["layers"], input_layer.name)) >= 0:
+                input_layer_name = config["layers"][layer_id]["config"]["name"]
+            else:
+                input_layer_name = None
+            insert_in_config(model, input_layer_name, observer, config)
 
     # Build the calibration model from the config
     calibration_model = model.from_config(config)
@@ -164,8 +157,13 @@ def _set_and_equalize(qmodel, layer_name, range_max):
     next_layer = _get_next_layer(target_layer, supported_layers, skippable_layers, reshape_ops)
     # For now cross layer equalization for (Dephwise)BufferTempConvLayers is possible only if their
     # input is quantized per-tensor.
-    if next_layer is None or (isinstance(next_layer, (BufferTempConv, DepthwiseBufferTempConv))
-                              and target_layer.out_quantizer.axis == 'per-axis'):
+    # cross layer equalization is not applied on conv2d when groups is greater than 1 and when
+    # data_format is 'channels_first'
+    if (next_layer is None
+        or (isinstance(next_layer, (BufferTempConv, DepthwiseBufferTempConv))
+            and target_layer.out_quantizer.axis == 'per-axis')
+            or getattr(next_layer, 'groups', 1) != 1
+            or getattr(next_layer, 'data_format', 'channels_last') == 'channels_first'):
         return
 
     # Compute ideal range_max
@@ -237,48 +235,37 @@ def _set_and_equalize(qmodel, layer_name, range_max):
     next_layer.set_weights(next_weights)
 
 
-def _warmup_calib(calibration_model, warmup_samples, end_of_blocks, batch_size):
-    """ Helper method to calibrate the buffer TENN models with warmpup period.
+def _set_and_fold_zero_point(qmodel, layer_name, range_min, range_max):
+    quantizer = qmodel.get_layer(layer_name)
 
-    Args:
-        calibration_model (keras.Model): the calibration model.
-        warmup_samples (tf.Dataset, np.array or generator, optional): warmup samples.
-        end_of_blocks (dict): dict mapping end of block layer names to the name of their
-            OutputObserver.
-        batch_size (int): calibration batch size.
-    """
+    # Update range_min if required.
+    if not quantizer.signed:
+        range_min = tf.where(range_min > 0.0, 0.0, range_min)
 
-    # Fill the fifos.
-    calibration_model.predict(x=warmup_samples, batch_size=batch_size)
-    # reset the range_max
-    for _, observer in end_of_blocks.items():
-        range_max = calibration_model.get_layer(observer).range_max
-        range_max.assign(tf.ones_like(range_max))
+    # Set ranges in quantizer.
+    quantizer.range_min.assign(range_min)
+    quantizer.range_max.assign(range_max)
 
+    # Stop procedure if all range_min are zero (zero point folding is not required).
+    if tf.reduce_all(range_min == 0.0) or quantizer.signed:
+        return qmodel
 
-def _split_samples(samples, buffer_layers, batch_size):
-    """Helper to split the samples into warmup and calibration samples for the TENN models
+    # Retrieve next_layer.
+    skippable_layers = (Rescaling,)
+    next_layer = quantizer
+    while (len(outbounds := next_layer.outbound_nodes) == 1 and
+           isinstance((next_layer := outbounds[0].layer), skippable_layers)):
+        # Update value to fold depending of next layer.
+        if isinstance(next_layer, Rescaling):
+            range_min = range_min * next_layer.scale
 
-    Args:
-        samples (np.array): samples.
-        buffer_layers (list: keras.Layers): the model (Depthwise)BufferTempConv layers
-        batch_size (int): calibration batch_size.
-
-    Returns:
-        np.array, np.array, int: warmup samples, effective calibration samples and warmup
-        steps.
-    """
-    # Extract warmup_factor. The number of samples required to fill all model fifos.
-    warmup_factor = 0
-    for buf_layer in buffer_layers:
-        warmup_factor += buf_layer.kernel_size
-    if warmup_factor * batch_size > samples.shape[0]:
-        raise ValueError(f"Number of samples must be greater than {warmup_factor} * batch_size.")
-    # Extract warmup samples
-    warmup_samples = samples[:warmup_factor * batch_size]
-    # Extract calibration samples
-    calib_samples = samples[warmup_factor * batch_size:]
-    return warmup_samples, calib_samples, warmup_factor
+    # Fold zero point into bias of next layer.
+    try:
+        fake_rescaling_layer = Rescaling(scale=1.0, offset=range_min)
+        _fold_rescaling(fake_rescaling_layer, next_layer)
+    except Exception as e:
+        raise RuntimeError("During calibration, impossible to fold zero point in "
+                           f"{next_layer.name}.") from e
 
 
 def calibrate(model, qmodel, samples=None, num_samples=1024, batch_size=None, epochs=1):
@@ -307,29 +294,18 @@ def calibrate(model, qmodel, samples=None, num_samples=1024, batch_size=None, ep
         epochs (int, optional): the number of epochs. Defaults to 1.
 
     """
-    if not calibration_required(qmodel):
-        return
-
-    # Disable sumbsampling in StatefulRecurrent layers before building the calibration model
-    rec_layers = get_layers_by_type(model, StatefulRecurrent)
-    for ly in rec_layers:
-        ly.subsample_ratio = None
-
-    # Build a calibration model which is a float model with OutputObservers at locations where the
-    # quantized model has OutputQuantizers.
+    # Build a calibration model which is a float model with Observers at locations where the
+    # quantized model has InputQuantizers/OutputQuantizers.
     calibration_model, end_of_blocks = _get_calibration_model(model, qmodel)
 
     if samples is None:
-        # Generate random samples
-        samples_shape = (num_samples, ) + qmodel.input_shape[1:]
-        samples_type = get_model_input_dtype(model).as_numpy_dtype
+        # Generate random samples.
+        # Note we use the float model since the quantized model has not yet been calibrated.
+        samples = generate_keras_random_samples(model, num_samples)
 
-        rng = np.random.default_rng()
-        iinfo = np.iinfo(samples_type)
-        samples = rng.integers(iinfo.min, iinfo.max, samples_shape, dtype=samples_type)
-
-    # Extract number of (Depthwise)BufferTempConv layers
+    # Extract number of (Depthwise)BufferTempConv and StatefulRecurrent layers
     buf_layers = get_layers_by_type(calibration_model, (BufferTempConv, DepthwiseBufferTempConv))
+    rec_layers = get_layers_by_type(calibration_model, StatefulRecurrent)
     if buf_layers or rec_layers:
         if not isinstance(samples, np.ndarray):
             raise TypeError("TENN models calibration is only possible with np.array samples."
@@ -367,6 +343,7 @@ def calibrate(model, qmodel, samples=None, num_samples=1024, batch_size=None, ep
 
     for i in range(epochs):
         if rec_layers:
+            calibration_model = update_batch_size(calibration_model, samples.shape[0])
             # Ensure graph execution
             model_func = tf.function(calibration_model)
             in_shape = calibration_model.input_shape[1]
@@ -379,19 +356,12 @@ def calibrate(model, qmodel, samples=None, num_samples=1024, batch_size=None, ep
             for i in range(in_step):
                 model_func(samples[:, i * in_shape:(i + 1) * in_shape])
         else:
-            # Set default calib samples and number of steps
-            calib_samples = samples
-            calibration_steps = steps
-            # If the model is a buffer TENN, it normally requires a warmup period on the first epoch
-            # in order to calibrate it. For that purpose batch_size is mandatory.
-            if len(buf_layers) > 0 and batch_size and i == 0:
-                warmup_samples, calib_samples, warmup_steps = _split_samples(samples, buf_layers,
-                                                                             batch_size)
-                _warmup_calib(calibration_model, warmup_samples, end_of_blocks, batch_size)
-                calibration_steps = steps - warmup_steps
             calibration_model.predict(
-                x=calib_samples, steps=calibration_steps, batch_size=batch_size)
+                x=samples, steps=steps, batch_size=batch_size)
 
-    # Update quantized model OutputQuantizers range_max using OutputObservers calibrated values
+    # Update quantized model calibration ranges using Observers calibrated values.
     for eob, observer in end_of_blocks.items():
-        _set_and_equalize(qmodel, eob, calibration_model.get_layer(observer).range_max)
+        if isinstance(observer := calibration_model.get_layer(observer), OutputObserver):
+            _set_and_equalize(qmodel, eob, observer.range_max)
+        else:
+            _set_and_fold_zero_point(qmodel, eob, observer.range_min, observer.range_max)
