@@ -20,28 +20,23 @@ __all__ = ["quantize", "dump_config"]
 import warnings
 from collections import defaultdict
 
+import tf_keras as keras
 import numpy as np
-import keras
+from tf_keras.saving import get_registered_object, serialize_keras_object
 from onnx import ModelProto
-try:
-    # Legacy support for keras 2.10
-    from keras.utils.generic_utils import get_registered_object, get_registered_name
-except ImportError:
-    from keras.saving.object_registration import get_registered_object, get_registered_name
 
-from .utils import apply_weights_to_model
+from ..layers import (Activation, Dequantizer, InputQuantizer, OutputQuantizer, QuantizationParams,
+                      QuantizedActivation, WeightQuantizer, get_quantization_params, quantization,
+                      StatefulRecurrent, BufferTempConv, StatefulProjection)
+from ..layers.layers_base import (_GLOBAL_ALIGNED_INPUTS, _GLOBAL_LAYER_TO_QLAYER,
+                                  _GLOBAL_NO_OUTPUT_QUANTIZER, _GLOBAL_QUANTIZABLE_WEIGHTS)
+from ..onnx_support.quantization.quantize import quantize as quantize_onnx
+from .calibrate import calibrate
+from .shape import set_model_shape
 from .transforms import sanitize
 from .transforms.insert_layer import insert_in_config
-from .transforms.transforms_utils import get_inbound_layers_config
-from .calibrate import calibrate
-from ..layers import (OutputQuantizer, WeightQuantizer, Dequantizer, quantization,
-                      QuantizationParams, get_quantization_params, Attention, QuantizedConv2D,
-                      QuantizedDepthwiseConv2D, QuantizedSeparableConv2D, QuantizedDense,
-                      StatefulRecurrent, QuantizedActivation)
-from ..layers.layers_base import (_GLOBAL_LAYER_TO_QLAYER, _GLOBAL_NO_OUTPUT_QUANTIZER,
-                                  _GLOBAL_ALIGNED_INPUTS)
-from ..onnx_support.quantization.quantize import quantize as quantize_onnx
-
+from .transforms.transforms_utils import get_inbound_layers_config, get_layer_index, get_layers
+from .utils import apply_weights_to_model, requires_tf_keras_model
 
 # List of Quantizer layer's that do not have a float layer representation
 NO_FLOAT_CUSTOM_QLAYERS = [Dequantizer, OutputQuantizer, WeightQuantizer]
@@ -57,17 +52,15 @@ def get_quantized_layer(layer):
         keras.layer: quantized version of the layer if it exists, None otherwise.
     """
     if isinstance(layer, keras.layers.Layer):
-        config = layer.get_config()
         layer_class = layer.__class__
     else:
-        config = layer['config']
-        layer_class = get_layer_class_from_name(layer["class_name"])
+        layer_class = get_layer_class_from_config(layer)
+    if layer_class is None:
+        return None
     qlayer_class = _GLOBAL_LAYER_TO_QLAYER.get(layer_class.__name__, None)
 
-    # Special case for activations: avoid quantization if activation is not within the
-    # allowed activations
-    allowed_activations = QuantizedActivation.arg_constraints['activation']()
-    if qlayer_class == QuantizedActivation and config['activation'] not in allowed_activations:
+    # Special case for activations: avoid quantization if layer_class corresponds to the keras one
+    if qlayer_class == QuantizedActivation and layer_class == keras.layers.Activation:
         return None
     return qlayer_class
 
@@ -76,28 +69,38 @@ def is_quantized_layer(layer):
     """ Returns True when the layer is a quantized layer.
 
     Args:
-        layer (keras.layers.Layer or type): layer of interest
+        layer (dict or keras.layers.Layer or type): layer of interest
 
     Returns:
         bool: True when the layer is a quantized layer, False otherwise.
     """
+    if isinstance(layer, dict):
+        layer = get_layer_class_from_config(layer)
     if isinstance(layer, type):
         return layer in _GLOBAL_LAYER_TO_QLAYER.values()
     return layer.__class__ in _GLOBAL_LAYER_TO_QLAYER.values()
 
 
-def get_layer_class_from_name(name):
+def get_layer_class_from_config(config):
     """ Returns the class object of a registered keras layer.
 
     Args:
-        name (str): the layer class name
+        config (dict): the layer config
 
     Returns:
         type: the class of the layer
     """
-    if hasattr(keras.layers, name):
-        return getattr(keras.layers, name)
-    return get_registered_object(name)
+    # First try to get registered custom object
+    if ly_class := get_registered_object(config.get("registered_name", None)):
+        return ly_class
+
+    # Check class name in different keras modules
+    class_name = config["class_name"]
+    for module in (keras.layers, keras.src.layers, keras):
+        if hasattr(module, class_name):
+            return getattr(module, class_name)
+
+    raise ValueError(f"Impossible to recognize layer with configuration {config}.")
 
 
 def _handle_not_quantizable_layers(model, config, skip_warning=False):
@@ -117,15 +120,12 @@ def _handle_not_quantizable_layers(model, config, skip_warning=False):
     num_no_quantizable_layers = 0
     dequantizer_inbounds = defaultdict(list)
     for layer in config['layers']:
-        layer_class = get_layer_class_from_name(layer['class_name'])
-        if layer_class == Dequantizer:
-            # If a model has a Dequantizer is because it is requantizing.
-            # In that case nothing has to be done (model already has the necessary dequantizers)
-            return False
-        elif not (is_quantized_layer(layer_class) or layer_class in NO_FLOAT_CUSTOM_QLAYERS):
+        layer_class = get_layer_class_from_config(layer)
+
+        if not (is_quantized_layer(layer_class) or layer_class in NO_FLOAT_CUSTOM_QLAYERS):
             # The current layer is float.
             for inbound_layer in get_inbound_layers_config(layer, config):
-                inbound_layer_class = get_layer_class_from_name(inbound_layer['class_name'])
+                inbound_layer_class = get_layer_class_from_config(inbound_layer)
                 # A dequantizer will be added for each quantized inbound layer
                 if is_quantized_layer(inbound_layer_class):
                     inbound_layer_name = inbound_layer['config']['name']
@@ -133,7 +133,8 @@ def _handle_not_quantizable_layers(model, config, skip_warning=False):
                         # Print warning for the first non-quantizable layer
                         warnings.warn(f"'{layer['config']['name']}' of type {layer['class_name']} "
                                       "is not supported to quantize, a Dequantizer is added "
-                                      "before it and quantization will stop at this layer.")
+                                      "before it and quantization will stop at this layer."
+                                      "Continuing execution.")
                     dequantizer_inbounds[inbound_layer_name].append(layer['config']['name'])
             num_no_quantizable_layers += 1
 
@@ -176,24 +177,14 @@ def _prepare_output_quantizers(model):
             current_layer = model.get_layer(name)
             # Handle special cases where the OutputQuantizer must be per-tensor:
             # - when current_layer has vector outputs,
-            # - when next_layer is an Attention layer and the layer is Query or Key
-            #   (first and second inputs)
             # - when the layer is a StatefulRecurrent layer
-            if isinstance(current_layer, Attention):
-                output_shape = current_layer.output_shape[0]
-            else:
-                output_shape = current_layer.output_shape
             # remove batch_size dim
-            output_shape = output_shape[1:]
+            output_shape = current_layer.output_shape[1:]
             vector_outputs = np.prod(output_shape) == output_shape[-1]
-            query_or_key = (isinstance(current_layer, keras.layers.Dense)
-                            and isinstance(next_layer, Attention)
-                            and next_layer.inbound_nodes[0].inbound_layers.
-                            index(current_layer) in [0, 1])
-            is_stateful_rec = isinstance(current_layer, StatefulRecurrent)
             is_next_activation = get_quantized_layer(next_layer) == QuantizedActivation
-            per_tensor = (query_or_key or vector_outputs or is_stateful_rec or
-                          is_next_activation or qparams.per_tensor_activations)
+            is_stateful_rec = isinstance(current_layer, StatefulRecurrent)
+            per_tensor = (vector_outputs or is_next_activation or qparams.per_tensor_activations or
+                          is_stateful_rec)
 
             # If this is a new entry, set a default configuration
             if name not in out_quantizer_configs:
@@ -202,10 +193,17 @@ def _prepare_output_quantizers(model):
                     params = dict(bitwidth=qparams.activation_bits,
                                   signed=qparams.activation_bits >= 8,
                                   axis=axis)
-                elif isinstance(current_layer, keras.layers.Activation):
+                elif isinstance(current_layer, Activation):
                     params = dict(bitwidth=qparams.activation_bits, axis=axis)
                 elif is_next_activation:
                     params = dict(bitwidth=QuantizedActivation.DEFAULT_INPUT_BITWIDTH, axis=axis)
+                elif isinstance(current_layer, keras.layers.GlobalAveragePooling2D):
+                    # Traverse back through Reshape and Dropout layers to find the previous layers
+                    previous_name = get_preceding_layer_names(current_layer)[0]
+                    if isinstance(model.get_layer(previous_name), (keras.layers.ReLU, Activation)):
+                        params = dict(bitwidth=qparams.activation_bits, axis=axis)
+                    else:
+                        params = dict(bitwidth=qparams.output_bits, axis=axis)
                 else:
                     # StatefulRecurrent special: previous and self OutputQuantizer should be 16-bits
                     if is_stateful_rec or isinstance(next_layer, StatefulRecurrent):
@@ -261,8 +259,94 @@ def _prepare_output_quantizers(model):
     return out_quantizer_configs
 
 
+def _apply_last_output_bits(qmodel_config, qparams):
+    output_layer_names = [out[0] for out in qmodel_config['output_layers']]
+    for layer_name in output_layer_names:
+        layer_index = get_layer_index(qmodel_config['layers'], layer_name)
+        layer_config = qmodel_config['layers'][layer_index]
+        layer_class = get_layer_class_from_config(layer_config)
+        if layer_class in _GLOBAL_NO_OUTPUT_QUANTIZER:
+            raise RuntimeError(f'Cannot apply last_output_bits on {layer_name}.')
+        out_quantizer_params = {'bitwidth': qparams.last_output_bits,
+                                'buffer_bitwidth': qparams.buffer_bits,
+                                'scale_bits': 16 if qparams.last_output_bits >= 16 else 8}
+        layer_config['config']['quant_config']['output_quantizer'] = out_quantizer_params
+
+
+def _insert_input_quantizer(model, qmodel_config):
+    """Inserts an InputQuantizer layer at the beginning of the model.
+
+    This is done only if model input dtype is floating.
+
+    Args:
+        model (keras.Model): the original Keras model.
+        qmodel_config (dict): the model configuration dictionary to be modified.
+    """
+    qparams = get_quantization_params()
+    common_kwargs = dict(bitwidth=8 * qparams.input_dtype.itemsize,
+                         signed=np.issubdtype(qparams.input_dtype, np.signedinteger),
+                         axis="per-tensor" if qparams.per_tensor_activations else "per-axis")
+
+    # Search for InputLayer layers in the model.
+    if isinstance(model, keras.Sequential):
+        input_layer = qmodel_config["layers"][0]
+        if get_layer_class_from_config(input_layer) != keras.layers.InputLayer:
+            # There is not an InputLayer in the model.
+            # Build a fake config to match with next procedure.
+            layer_config = {k: v for k, v in input_layer["config"].items()
+                            if k in ["dtype", "batch_input_shape"]}
+            input_layer = {"config": {"name": None, **layer_config}}
+    else:
+        # InputQuantizer will be inserted in the first input layer.
+        input_layer_names = [ilayer[0] for ilayer in qmodel_config['input_layers']]
+        input_layer = get_layers(qmodel_config, input_layer_names)[0]
+
+    # Insert an InputQuantizer as the outbound of input layer.
+    # Note this happens iff the layer dtype is float.
+    input_dtype = np.dtype(input_layer["config"]["dtype"])
+    input_quantizer = None
+    if np.issubdtype(input_dtype, np.floating):
+        # We explictly specify the input shape in InputQuantizer to ensure that the
+        # model always compiles when calling model.from_config(...).
+        input_shape = input_layer["config"]["batch_input_shape"][1:]
+        # Overwrite axis if input_shape is 2D.
+        if len(input_shape) < 2:
+            common_kwargs["axis"] = "per-tensor"
+        input_quantizer = InputQuantizer(input_shape=input_shape, **common_kwargs)
+        insert_in_config(model, input_layer["config"]["name"], input_quantizer, qmodel_config)
+    elif input_dtype != qparams.input_dtype:
+        warnings.warn(f"The provided model expects {input_dtype} in "
+                      f"{input_layer['config']['name']} but QuantizationParams.input_dtype is "
+                      f"{input_dtype} which is different. Keeping original signature without "
+                      "adding an input quantizer and continuing execution.")
+
+    # Adapt InputQuantizer outbound layer. Note this only happens when signed = False.
+    if input_quantizer is not None and not input_quantizer.signed:
+        # Reject any InputQuantizer outbound that does not support a QFloat as input.
+        skippable_layers = (keras.layers.Rescaling, keras.layers.InputLayer)
+        supported_layers = (keras.layers.Conv2D, BufferTempConv, keras.layers.Dense,
+                            StatefulProjection)
+        next_layer = model.layers[0]
+        while (len(outbounds := next_layer.outbound_nodes) == 1 and
+               isinstance(next_layer, skippable_layers)):
+            next_layer = outbounds[0].layer
+        if not isinstance(next_layer, supported_layers):
+            raise ValueError("Impossible to quantize inputs: unsigned quantization "
+                             f"(input_dtype = {qparams.input_dtype}) is only possible if "
+                             f"first layer ({next_layer.name}) was one of {supported_layers}.")
+        # Adapt outbound to support fold zero point when calibrating.
+        layer_config = get_layers(qmodel_config, [next_layer.name])[0]["config"]
+        layer_config["use_bias"] = True
+        layer_config.pop("bias_initializer", None)
+        if (isinstance(next_layer, keras.layers.Conv2D) and layer_config["padding"] == "same"
+                and layer_config.get("padding_value", None) is None):
+            layer_config["padding_value"] = [0.0]
+
+
+@requires_tf_keras_model
 def quantize_keras(model, q_config=None, qparams=QuantizationParams(), samples=None,
-                   num_samples=1024, batch_size=None, epochs=1, quantize_until=None):
+                   num_samples=1024, batch_size=None, epochs=1, quantize_until=None,
+                   input_shape=None):
     """Quantizes a Keras model using the provided configuration or parameters.
 
     Details on how this function behaves:
@@ -294,10 +378,24 @@ def quantize_keras(model, q_config=None, qparams=QuantizationParams(), samples=N
         epochs (int, optional): the number of epochs. Defaults to 1.
         quantize_until (str, optional): name of the layer until which to quantize:
             other layers after it will stay unchanged. Defaults to None.
+        input_shape (list or tuple, optional): Specifies the new input shape for the model.
+            in the format of (H, W, C) for 4D inputs or (T, H, W, C) for 5D inputs.
+            Used only if `samples` is not provided. Defaults to None.
 
     Returns:
         keras.Model: the quantized model
     """
+    # Prevent requantization
+    if any(is_quantized_layer(layer) for layer in model.layers):
+        raise ValueError("Requantizing a model is not supported. "
+                         "Please quantize the original float model directly.")
+
+    # Check for unsupported layers before running the quantization pipeline
+    try:
+        model.from_config(model.get_config())
+    except Exception as e:
+        raise ValueError("Unserializable layers were found in the model.") from e
+
     q_config = q_config or dict()
     if quantize_until and not any(ly.name == quantize_until for ly in model.layers):
         raise ValueError(f"'{quantize_until}' is not a recognized layer in {model.name}")
@@ -309,6 +407,7 @@ def quantize_keras(model, q_config=None, qparams=QuantizationParams(), samples=N
                                        per_tensor_activations=qparams.per_tensor_activations,
                                        weight_bits=qparams.input_weight_bits,
                                        output_bits=qparams.output_bits,
+                                       input_dtype=qparams.input_dtype,
                                        buffer_bits=qparams.buffer_bits)
 
     def get_quantize_layer(layer, quantize_config=None):
@@ -317,7 +416,7 @@ def quantize_keras(model, q_config=None, qparams=QuantizationParams(), samples=N
             - then, we return the quantized layer with config updated
         """
         # Check if qlayer exists in custom layers and returns the float version of the layer if not
-        l_class = get_layer_class_from_name(layer["class_name"])
+        l_class = get_layer_class_from_config(layer)
         ql_class = get_quantized_layer(layer)
         if ql_class is None:
             ql_class = l_class
@@ -332,14 +431,21 @@ def quantize_keras(model, q_config=None, qparams=QuantizationParams(), samples=N
         elif qlayer['config']['name'] in out_quantizer_configs:
             qlayer['config']['quant_config'] = out_quantizer_configs[qlayer['config']['name']]
 
-        # Retrieve the quantize config after initializing the quantized layer, in order to
+        # Retrieve the quantized config after initializing the quantized layer, in order to
         # configure the specific parameters given by the QuantizationParams context.
-        qlayer['config'] = ql_class.from_config(qlayer['config']).get_config()
-
-        # Update layer class name by the quantized one
-        qlayer['class_name'] = get_registered_name(ql_class)
+        # If the quantized layer initialization raises an error, initialize it with the float
+        # layer config
+        try:
+            new_layer = ql_class.from_config(qlayer['config'])
+        except TypeError:
+            qlayer['config'].pop('quant_config', None)
+            new_layer = l_class.from_config(qlayer['config'])
+        qlayer.update(serialize_keras_object(new_layer))
         return qlayer
 
+    # Set model shape if needed
+    model = set_model_shape(model, samples, input_shape)
+
     # Sanitize the model and make it quantization ready
     model = sanitize(model)
 
@@ -349,17 +455,9 @@ def quantize_keras(model, q_config=None, qparams=QuantizationParams(), samples=N
         out_quantizer_configs = _prepare_output_quantizers(model)
 
     # Quantize the model, modifying each layer config by its respective quantized version
-    input_layers = (QuantizedConv2D, QuantizedDepthwiseConv2D, QuantizedSeparableConv2D,
-                    QuantizedDense)
     qmodel_config = model.get_config()
     quantized_layers = set()
     for idx, layer in enumerate(qmodel_config['layers']):
-        # If some layer is quantized then this is requantization.
-        # Raise exception for this case if quantize_until is provided
-        layer_class = get_layer_class_from_name(layer['class_name'])
-        if is_quantized_layer(layer_class) and quantize_until:
-            raise ValueError("'quantize_until' is not supported when requantizing.")
-
         # Retrieve quantize config from layer
         match_conf = q_config.get(layer['config']['name'], None)
 
@@ -372,9 +470,9 @@ def quantize_keras(model, q_config=None, qparams=QuantizationParams(), samples=N
             else:
                 qlayer = layer
 
-        # When the qlayer is an input layer that has been quantized, disable input_qparams
-        qlayer_class = get_layer_class_from_name(qlayer['class_name'])
-        if input_qparams and qlayer_class in input_layers:
+        # Disable input_qparams when a layer with weights has already been quantized
+        qlayer_class = get_layer_class_from_config(qlayer)
+        if input_qparams and qlayer_class in _GLOBAL_QUANTIZABLE_WEIGHTS:
             input_qparams = None
 
         # Skip input layers
@@ -398,11 +496,21 @@ def quantize_keras(model, q_config=None, qparams=QuantizationParams(), samples=N
             # Note if layer is within a branch, quantization will end only for this branch
             quantized_layers.add(qlayer['config']['name'])
 
+    # Input quantization: add the required InputQuantizer(s).
+    # Note this happens iff there is at least one quantized layer.
+    with quantization(qparams):
+        if any(is_quantized_layer(layer) for layer in qmodel_config['layers']):
+            _insert_input_quantizer(model, qmodel_config)
+
     # Insert the number of Dequantizers necessary for the model to be compatible
     is_full_quantized = _handle_not_quantizable_layers(model,
                                                        qmodel_config,
                                                        skip_warning=quantize_until is not None)
 
+    # Apply last_output_bits to fully quantized models
+    if is_full_quantized and qparams.last_output_bits:
+        _apply_last_output_bits(qmodel_config, qparams)
+
     # Build the model and transfer weights
     qmodel = model.from_config(qmodel_config)
     apply_weights_to_model(qmodel, {var.name: var for var in model.variables}, False)
@@ -415,13 +523,14 @@ def quantize_keras(model, q_config=None, qparams=QuantizationParams(), samples=N
     qmodel = keras.Model(qmodel.input, y, name=model.name)
 
     # Now that the model is quantized, proceed to calibration
-    calibrate(model, qmodel, samples=samples, num_samples=num_samples, batch_size=batch_size,
-              epochs=epochs)
+    with quantization(qparams):
+        calibrate(model, qmodel, samples=samples, num_samples=num_samples, batch_size=batch_size,
+                  epochs=epochs)
     return qmodel
 
 
 def quantize(model, q_config=None, qparams=QuantizationParams(), samples=None, num_samples=1024,
-             batch_size=None, epochs=1, quantize_until=None):
+             batch_size=None, epochs=1, quantize_until=None, input_shape=None):
     """Quantizes a Keras or ONNX model using the provided configuration or parameters.
 
     Details on how this function behaves:
@@ -456,6 +565,8 @@ def quantize(model, q_config=None, qparams=QuantizationParams(), samples=None, n
             Defaults to 1.
         quantize_until (str, optional): name of the layer/node until which to quantize:
             other ones after it will stay unchanged. Defaults to None.
+        input_shape (list or tuple, optional): A list or tuple specifying the new model input shape
+            excluding batch dimension. Defaults to None.
 
     Returns:
         keras.Model or ModelProto: the quantized model
@@ -463,9 +574,9 @@ def quantize(model, q_config=None, qparams=QuantizationParams(), samples=None, n
     # Calibration with random samples will only provide meaningful results when quantizing
     # per-tensor
     if samples is None and not qparams.per_tensor_activations:
-        warnings.warn("Quantizing per-axis with random calibration samples is not accurate.\
-                       Set QuantizationParams.per_tensor_activations=True when calibrating with \
-                       random samples.")
+        warnings.warn("Quantizing per-axis with random calibration samples is not accurate. "
+                      "Set QuantizationParams.per_tensor_activations=True when calibrating with "
+                      "random samples. Continuing execution.")
     if type(model) != ModelProto:
         return quantize_keras(model=model,
                               q_config=q_config,
@@ -474,7 +585,8 @@ def quantize(model, q_config=None, qparams=QuantizationParams(), samples=None, n
                               num_samples=num_samples,
                               batch_size=batch_size,
                               epochs=epochs,
-                              quantize_until=quantize_until)
+                              quantize_until=quantize_until,
+                              input_shape=input_shape)
     elif q_config:
         raise ValueError("unsupported parameter q_config for ONNX models quantization")
     elif epochs != 1:
@@ -484,7 +596,8 @@ def quantize(model, q_config=None, qparams=QuantizationParams(), samples=None, n
                          samples=samples,
                          num_samples=num_samples,
                          batch_size=batch_size,
-                         quantize_until=quantize_until)
+                         quantize_until=quantize_until,
+                         input_shape=input_shape)
 
 
 def dump_config(model):
