@@ -20,21 +20,24 @@ Transformation to fold BatchNormalization layers in a model.
 
 __all__ = ['fold_batchnorms']
 
-import keras
+import tf_keras as keras
 import numpy as np
 import tensorflow as tf
 
 from copy import deepcopy
-from keras.layers import (BatchNormalization, Conv2D, SeparableConv2D, Dense, DepthwiseConv2D,
-                          Conv2DTranspose)
+from tf_keras.layers import (BatchNormalization, Conv2D, SeparableConv2D, Dense, DepthwiseConv2D,
+                             Conv2DTranspose)
 
-from .transforms_utils import get_layers, get_layer_index, update_inbound, find_layers_pairs
-from ..utils import apply_weights_to_model
+from .transforms_utils import (get_layers, get_layer_index, update_inbound, find_layers_pairs,
+                               safe_fail)
+from ..utils import apply_weights_to_model, requires_tf_keras_model
 from ...layers import (PaddedConv2D, QuantizedConv2D, QuantizedSeparableConv2D, QuantizedDense,
                        QuantizedDepthwiseConv2D, QuantizedConv2DTranspose, DepthwiseConv2DTranspose,
                        QuantizedDepthwiseConv2DTranspose, BufferTempConv, DepthwiseBufferTempConv)
 
 
+@safe_fail
+@requires_tf_keras_model
 def fold_batchnorms(model):
     """Returns a new model where BatchNormalization layers are folded into previous layers.
 
