@@ -22,10 +22,12 @@ __all__ = ["remove_zeropadding2d"]
 
 from copy import deepcopy
 
-from keras.models import Sequential
-from keras.layers import ZeroPadding2D, Conv2D, SeparableConv2D, DepthwiseConv2D
+from tf_keras.models import Sequential
+from tf_keras.layers import ZeroPadding2D, Conv2D, SeparableConv2D, DepthwiseConv2D
 
-from .transforms_utils import get_layers, get_layer_index, get_layers_by_type, update_inbound
+from .transforms_utils import (get_layers, get_layer_index, get_layers_by_type, update_inbound,
+                               safe_fail)
+from ..utils import requires_tf_keras_model
 
 
 def _find_removable_zeropadding(model):
@@ -48,10 +50,6 @@ def _find_removable_zeropadding(model):
     # Get all ZeroPadding2D layers present in the model
     zeropaddings = get_layers_by_type(model, ZeroPadding2D)
 
-    # Sequential models must be built because a check must be done on shapes
-    if zeropaddings and isinstance(model, Sequential) and not model.built:
-        raise ValueError("This model has not yet been built.")
-
     # Find the ones that can be removed
     for zeropadding in zeropaddings:
         # Limit support to single inbound/outbound
@@ -124,6 +122,8 @@ def _get_zeropadding_less_model(model, map_zeropadding_next):
     return model.from_config(config)
 
 
+@safe_fail
+@requires_tf_keras_model
 def remove_zeropadding2d(model):
     """ Removes ZeroPadding2D layers from a model.
 
