@@ -53,6 +53,9 @@ class FixedPoint(QTensor):
             self.frac_bits = tf.cast(frac_bits, tf.float32)
         else:
             self.frac_bits = tf.convert_to_tensor(frac_bits, tf.float32)
+        if tf.squeeze(self.frac_bits).shape.ndims > 1:
+            raise ValueError("The frac_bits must be a scalar or a one-dimensional array. "
+                             "Found an array with shape: {}".format(self.frac_bits.shape))
         self.value_bits = value_bits
 
         self.shape = self.values.shape
