@@ -252,8 +252,7 @@ def slice_tensor_by_index(idx, values_rank, tensor):
     Returns:
         tf.Tensor: the sliced frac_bits/scale tensor tensor.
     """
-
-    tensor_rank = tf.rank(tensor)
+    assert_equal(tf.rank(tensor), 1)
     if isinstance(idx, tuple):
         # Convert Ellipsis into actual slices
         if any(x is Ellipsis for x in idx):
@@ -261,23 +260,23 @@ def slice_tensor_by_index(idx, values_rank, tensor):
 
         # Case where the number of indices matches the rank of the values tensor
         if values_rank == len(idx):
-            new_fb_tuple = idx[-tensor_rank:]
-            return tensor[new_fb_tuple]
+            # frac_bits/scale have maximum 1 dimension so the slicing is done on the last dimension.
+            return tensor[idx[-1]]
 
         # Case where the frac_bits/scales tensor has fewer dimensions than the remaining positions
-        elif values_rank - len(idx) - tensor_rank >= 0:
+        elif values_rank - len(idx) - 1 >= 0:
             return tensor
 
         # Case where the frac_bits/scales tensor needs more dimensions
         else:
-            m = tensor_rank - (values_rank - len(idx))
+            m = 1 - (values_rank - len(idx))
             fb_ind = idx[-m:]
             fb_ind += (Ellipsis,)
             return tensor[fb_ind]
 
     # Case where the index is not a tuple (int, ellipsis, scalar)
     # and values values has more dimensions than the frac_bits/scales tensor
-    elif values_rank > tensor_rank:
+    elif values_rank > 1:
         return tensor
 
     # Case where the values and tensor have the same dimensions
